# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.url_normalization import url_normalization_update_params
from ...types.url_normalization.url_normalization_get_response import URLNormalizationGetResponse
from ...types.url_normalization.url_normalization_update_response import URLNormalizationUpdateResponse

__all__ = ["URLNormalizationResource", "AsyncURLNormalizationResource"]


class URLNormalizationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> URLNormalizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return URLNormalizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> URLNormalizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return URLNormalizationResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        zone_id: str,
        scope: Literal["incoming", "both", "none"],
        type: Literal["cloudflare", "rfc3986"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> URLNormalizationUpdateResponse:
        """
        Updates the URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          scope: The scope of the URL normalization.

          type: The type of URL normalization performed by Cloudflare.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            body=maybe_transform(
                {
                    "scope": scope,
                    "type": type,
                },
                url_normalization_update_params.URLNormalizationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[URLNormalizationUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[URLNormalizationUpdateResponse], ResultWrapper[URLNormalizationUpdateResponse]),
        )

    def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes the URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> URLNormalizationGetResponse:
        """
        Fetches the current URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[URLNormalizationGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[URLNormalizationGetResponse], ResultWrapper[URLNormalizationGetResponse]),
        )


class AsyncURLNormalizationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncURLNormalizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncURLNormalizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncURLNormalizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncURLNormalizationResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        zone_id: str,
        scope: Literal["incoming", "both", "none"],
        type: Literal["cloudflare", "rfc3986"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> URLNormalizationUpdateResponse:
        """
        Updates the URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          scope: The scope of the URL normalization.

          type: The type of URL normalization performed by Cloudflare.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "scope": scope,
                    "type": type,
                },
                url_normalization_update_params.URLNormalizationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[URLNormalizationUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[URLNormalizationUpdateResponse], ResultWrapper[URLNormalizationUpdateResponse]),
        )

    async def delete(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes the URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> URLNormalizationGetResponse:
        """
        Fetches the current URL Normalization settings.

        Args:
          zone_id: The unique ID of the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/url_normalization", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[URLNormalizationGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[URLNormalizationGetResponse], ResultWrapper[URLNormalizationGetResponse]),
        )


class URLNormalizationResourceWithRawResponse:
    def __init__(self, url_normalization: URLNormalizationResource) -> None:
        self._url_normalization = url_normalization

        self.update = to_raw_response_wrapper(
            url_normalization.update,
        )
        self.delete = to_raw_response_wrapper(
            url_normalization.delete,
        )
        self.get = to_raw_response_wrapper(
            url_normalization.get,
        )


class AsyncURLNormalizationResourceWithRawResponse:
    def __init__(self, url_normalization: AsyncURLNormalizationResource) -> None:
        self._url_normalization = url_normalization

        self.update = async_to_raw_response_wrapper(
            url_normalization.update,
        )
        self.delete = async_to_raw_response_wrapper(
            url_normalization.delete,
        )
        self.get = async_to_raw_response_wrapper(
            url_normalization.get,
        )


class URLNormalizationResourceWithStreamingResponse:
    def __init__(self, url_normalization: URLNormalizationResource) -> None:
        self._url_normalization = url_normalization

        self.update = to_streamed_response_wrapper(
            url_normalization.update,
        )
        self.delete = to_streamed_response_wrapper(
            url_normalization.delete,
        )
        self.get = to_streamed_response_wrapper(
            url_normalization.get,
        )


class AsyncURLNormalizationResourceWithStreamingResponse:
    def __init__(self, url_normalization: AsyncURLNormalizationResource) -> None:
        self._url_normalization = url_normalization

        self.update = async_to_streamed_response_wrapper(
            url_normalization.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            url_normalization.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            url_normalization.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .geo_restrictions import GeoRestrictions
from ..custom_hostnames.bundle_method import BundleMethod
from ..keyless_certificates.keyless_certificate import KeylessCertificate

__all__ = ["CustomCertificate"]


class CustomCertificate(BaseModel):
    id: str
    """Identifier."""

    zone_id: str
    """Identifier."""

    bundle_method: Optional[BundleMethod] = None
    """
    A ubiquitous bundle has the highest probability of being verified everywhere,
    even by clients using outdated or unusual trust stores. An optimal bundle uses
    the shortest chain and newest intermediates. And the force bundle verifies the
    chain, but does not otherwise modify it.
    """

    custom_csr_id: Optional[str] = None
    """The identifier for the Custom CSR that was used."""

    expires_on: Optional[datetime] = None
    """When the certificate from the authority expires."""

    geo_restrictions: Optional[GeoRestrictions] = None
    """
    Specify the region where your private key can be held locally for optimal TLS
    performance. HTTPS connections to any excluded data center will still be fully
    encrypted, but will incur some latency while Keyless SSL is used to complete the
    handshake with the nearest allowed data center. Options allow distribution to
    only to U.S. data centers, only to E.U. data centers, or only to highest
    security data centers. Default distribution is to all Cloudflare datacenters,
    for optimal performance.
    """

    hosts: Optional[List[str]] = None

    issuer: Optional[str] = None
    """The certificate authority that issued the certificate."""

    keyless_server: Optional[KeylessCertificate] = None

    modified_on: Optional[datetime] = None
    """When the certificate was last modified."""

    policy_restrictions: Optional[str] = None
    """The policy restrictions returned by the API.

    This field is returned in responses when a policy has been set. The API accepts
    the "policy" field in requests but returns this field as "policy_restrictions"
    in responses.

    Specifies the region(s) where your private key can be held locally for optimal
    TLS performance. Format is a boolean expression, for example: "(country: US) or
    (region: EU)"
    """

    priority: Optional[float] = None
    """The order/priority in which the certificate will be used in a request.

    The higher priority will break ties across overlapping 'legacy_custom'
    certificates, but 'legacy_custom' certificates will always supercede
    'sni_custom' certificates.
    """

    signature: Optional[str] = None
    """The type of hash used for the certificate."""

    status: Optional[Literal["active", "expired", "deleted", "pending", "initializing"]] = None
    """Status of the zone's custom SSL."""

    uploaded_on: Optional[datetime] = None
    """When the certificate was uploaded to Cloudflare."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.dns.settings.account import view_edit_params, view_list_params, view_create_params
from .....types.dns.settings.account.view_get_response import ViewGetResponse
from .....types.dns.settings.account.view_edit_response import ViewEditResponse
from .....types.dns.settings.account.view_list_response import ViewListResponse
from .....types.dns.settings.account.view_create_response import ViewCreateResponse
from .....types.dns.settings.account.view_delete_response import ViewDeleteResponse

__all__ = ["ViewsResource", "AsyncViewsResource"]


class ViewsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ViewsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ViewsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ViewsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ViewsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        zones: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewCreateResponse]:
        """
        Create Internal DNS View for an account

        Args:
          account_id: Identifier.

          name: The name of the view.

          zones: The list of zones linked to this view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dns_settings/views", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "zones": zones,
                },
                view_create_params.ViewCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewCreateResponse]], ResultWrapper[ViewCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: view_list_params.Name | Omit = omit,
        order: Literal["name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        zone_id: str | Omit = omit,
        zone_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ViewListResponse]:
        """
        List DNS Internal Views for an Account

        Args:
          account_id: Identifier.

          direction: Direction to order DNS views in.

          match: Whether to match all search requirements or at least one (any). If set to `all`,
              acts like a logical AND between filters. If set to `any`, acts like a logical OR
              instead.

          order: Field to order DNS views by.

          page: Page number of paginated results.

          per_page: Number of DNS views per page.

          zone_id: A zone ID that exists in the zones list for the view.

          zone_name: A zone name that exists in the zones list for the view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dns_settings/views", account_id=account_id),
            page=SyncV4PagePaginationArray[ViewListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "zone_id": zone_id,
                        "zone_name": zone_name,
                    },
                    view_list_params.ViewListParams,
                ),
            ),
            model=ViewListResponse,
        )

    def delete(
        self,
        view_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewDeleteResponse]:
        """
        Delete an existing Internal DNS View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewDeleteResponse]], ResultWrapper[ViewDeleteResponse]),
        )

    def edit(
        self,
        view_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        zones: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewEditResponse]:
        """
        Update an existing Internal DNS View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          name: The name of the view.

          zones: The list of zones linked to this view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "zones": zones,
                },
                view_edit_params.ViewEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewEditResponse]], ResultWrapper[ViewEditResponse]),
        )

    def get(
        self,
        view_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewGetResponse]:
        """
        Get DNS Internal View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewGetResponse]], ResultWrapper[ViewGetResponse]),
        )


class AsyncViewsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncViewsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncViewsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncViewsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncViewsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        zones: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewCreateResponse]:
        """
        Create Internal DNS View for an account

        Args:
          account_id: Identifier.

          name: The name of the view.

          zones: The list of zones linked to this view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dns_settings/views", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "zones": zones,
                },
                view_create_params.ViewCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewCreateResponse]], ResultWrapper[ViewCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        name: view_list_params.Name | Omit = omit,
        order: Literal["name", "created_on", "modified_on"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        zone_id: str | Omit = omit,
        zone_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ViewListResponse, AsyncV4PagePaginationArray[ViewListResponse]]:
        """
        List DNS Internal Views for an Account

        Args:
          account_id: Identifier.

          direction: Direction to order DNS views in.

          match: Whether to match all search requirements or at least one (any). If set to `all`,
              acts like a logical AND between filters. If set to `any`, acts like a logical OR
              instead.

          order: Field to order DNS views by.

          page: Page number of paginated results.

          per_page: Number of DNS views per page.

          zone_id: A zone ID that exists in the zones list for the view.

          zone_name: A zone name that exists in the zones list for the view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dns_settings/views", account_id=account_id),
            page=AsyncV4PagePaginationArray[ViewListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "name": name,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                        "zone_id": zone_id,
                        "zone_name": zone_name,
                    },
                    view_list_params.ViewListParams,
                ),
            ),
            model=ViewListResponse,
        )

    async def delete(
        self,
        view_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewDeleteResponse]:
        """
        Delete an existing Internal DNS View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewDeleteResponse]], ResultWrapper[ViewDeleteResponse]),
        )

    async def edit(
        self,
        view_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        zones: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewEditResponse]:
        """
        Update an existing Internal DNS View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          name: The name of the view.

          zones: The list of zones linked to this view.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "zones": zones,
                },
                view_edit_params.ViewEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewEditResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewEditResponse]], ResultWrapper[ViewEditResponse]),
        )

    async def get(
        self,
        view_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ViewGetResponse]:
        """
        Get DNS Internal View

        Args:
          account_id: Identifier.

          view_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not view_id:
            raise ValueError(f"Expected a non-empty value for `view_id` but received {view_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dns_settings/views/{view_id}", account_id=account_id, view_id=view_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ViewGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ViewGetResponse]], ResultWrapper[ViewGetResponse]),
        )


class ViewsResourceWithRawResponse:
    def __init__(self, views: ViewsResource) -> None:
        self._views = views

        self.create = to_raw_response_wrapper(
            views.create,
        )
        self.list = to_raw_response_wrapper(
            views.list,
        )
        self.delete = to_raw_response_wrapper(
            views.delete,
        )
        self.edit = to_raw_response_wrapper(
            views.edit,
        )
        self.get = to_raw_response_wrapper(
            views.get,
        )


class AsyncViewsResourceWithRawResponse:
    def __init__(self, views: AsyncViewsResource) -> None:
        self._views = views

        self.create = async_to_raw_response_wrapper(
            views.create,
        )
        self.list = async_to_raw_response_wrapper(
            views.list,
        )
        self.delete = async_to_raw_response_wrapper(
            views.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            views.edit,
        )
        self.get = async_to_raw_response_wrapper(
            views.get,
        )


class ViewsResourceWithStreamingResponse:
    def __init__(self, views: ViewsResource) -> None:
        self._views = views

        self.create = to_streamed_response_wrapper(
            views.create,
        )
        self.list = to_streamed_response_wrapper(
            views.list,
        )
        self.delete = to_streamed_response_wrapper(
            views.delete,
        )
        self.edit = to_streamed_response_wrapper(
            views.edit,
        )
        self.get = to_streamed_response_wrapper(
            views.get,
        )


class AsyncViewsResourceWithStreamingResponse:
    def __init__(self, views: AsyncViewsResource) -> None:
        self._views = views

        self.create = async_to_streamed_response_wrapper(
            views.create,
        )
        self.list = async_to_streamed_response_wrapper(
            views.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            views.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            views.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            views.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .directory import (
    DirectoryResource,
    AsyncDirectoryResource,
    DirectoryResourceWithRawResponse,
    AsyncDirectoryResourceWithRawResponse,
    DirectoryResourceWithStreamingResponse,
    AsyncDirectoryResourceWithStreamingResponse,
)
from .connectivity import (
    ConnectivityResource,
    AsyncConnectivityResource,
    ConnectivityResourceWithRawResponse,
    AsyncConnectivityResourceWithRawResponse,
    ConnectivityResourceWithStreamingResponse,
    AsyncConnectivityResourceWithStreamingResponse,
)

__all__ = [
    "DirectoryResource",
    "AsyncDirectoryResource",
    "DirectoryResourceWithRawResponse",
    "AsyncDirectoryResourceWithRawResponse",
    "DirectoryResourceWithStreamingResponse",
    "AsyncDirectoryResourceWithStreamingResponse",
    "ConnectivityResource",
    "AsyncConnectivityResource",
    "ConnectivityResourceWithRawResponse",
    "AsyncConnectivityResourceWithRawResponse",
    "ConnectivityResourceWithStreamingResponse",
    "AsyncConnectivityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from .members import (
    MembersResource,
    AsyncMembersResource,
    MembersResourceWithRawResponse,
    AsyncMembersResourceWithRawResponse,
    MembersResourceWithStreamingResponse,
    AsyncMembersResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.iam import user_group_list_params, user_group_create_params, user_group_update_params
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.iam.user_group_get_response import UserGroupGetResponse
from ....types.iam.user_group_list_response import UserGroupListResponse
from ....types.iam.user_group_create_response import UserGroupCreateResponse
from ....types.iam.user_group_delete_response import UserGroupDeleteResponse
from ....types.iam.user_group_update_response import UserGroupUpdateResponse

__all__ = ["UserGroupsResource", "AsyncUserGroupsResource"]


class UserGroupsResource(SyncAPIResource):
    @cached_property
    def members(self) -> MembersResource:
        return MembersResource(self._client)

    @cached_property
    def with_raw_response(self) -> UserGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return UserGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return UserGroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        policies: Iterable[user_group_create_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupCreateResponse]:
        """
        Create a new user group under the specified account.

        Args:
          account_id: Account identifier tag.

          name: Name of the User group.

          policies: Policies attached to the User group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/iam/user_groups", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                },
                user_group_create_params.UserGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupCreateResponse]], ResultWrapper[UserGroupCreateResponse]),
        )

    def update(
        self,
        user_group_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        policies: Iterable[user_group_update_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupUpdateResponse]:
        """
        Modify an existing user group.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          name: Name of the User group.

          policies: Policies attached to the User group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                },
                user_group_update_params.UserGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupUpdateResponse]], ResultWrapper[UserGroupUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        fuzzy_name: str | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[UserGroupListResponse]:
        """
        List all the user groups for an account.

        Args:
          account_id: Account identifier tag.

          id: ID of the user group to be fetched.

          direction: The sort order of returned user groups by name (ascending or descending).

          fuzzy_name: A string used for searching for user groups containing that substring.

          name: Name of the user group to be fetched.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/iam/user_groups", account_id=account_id),
            page=SyncV4PagePaginationArray[UserGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "direction": direction,
                        "fuzzy_name": fuzzy_name,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                    },
                    user_group_list_params.UserGroupListParams,
                ),
            ),
            model=UserGroupListResponse,
        )

    def delete(
        self,
        user_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupDeleteResponse]:
        """
        Remove a user group from an account.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupDeleteResponse]], ResultWrapper[UserGroupDeleteResponse]),
        )

    def get(
        self,
        user_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupGetResponse]:
        """
        Get information about a specific user group in an account.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupGetResponse]], ResultWrapper[UserGroupGetResponse]),
        )


class AsyncUserGroupsResource(AsyncAPIResource):
    @cached_property
    def members(self) -> AsyncMembersResource:
        return AsyncMembersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncUserGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncUserGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        policies: Iterable[user_group_create_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupCreateResponse]:
        """
        Create a new user group under the specified account.

        Args:
          account_id: Account identifier tag.

          name: Name of the User group.

          policies: Policies attached to the User group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/iam/user_groups", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                },
                user_group_create_params.UserGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupCreateResponse]], ResultWrapper[UserGroupCreateResponse]),
        )

    async def update(
        self,
        user_group_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        policies: Iterable[user_group_update_params.Policy] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupUpdateResponse]:
        """
        Modify an existing user group.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          name: Name of the User group.

          policies: Policies attached to the User group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "policies": policies,
                },
                user_group_update_params.UserGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupUpdateResponse]], ResultWrapper[UserGroupUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        id: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        fuzzy_name: str | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UserGroupListResponse, AsyncV4PagePaginationArray[UserGroupListResponse]]:
        """
        List all the user groups for an account.

        Args:
          account_id: Account identifier tag.

          id: ID of the user group to be fetched.

          direction: The sort order of returned user groups by name (ascending or descending).

          fuzzy_name: A string used for searching for user groups containing that substring.

          name: Name of the user group to be fetched.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/iam/user_groups", account_id=account_id),
            page=AsyncV4PagePaginationArray[UserGroupListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "direction": direction,
                        "fuzzy_name": fuzzy_name,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                    },
                    user_group_list_params.UserGroupListParams,
                ),
            ),
            model=UserGroupListResponse,
        )

    async def delete(
        self,
        user_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupDeleteResponse]:
        """
        Remove a user group from an account.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupDeleteResponse]], ResultWrapper[UserGroupDeleteResponse]),
        )

    async def get(
        self,
        user_group_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[UserGroupGetResponse]:
        """
        Get information about a specific user group in an account.

        Args:
          account_id: Account identifier tag.

          user_group_id: User Group identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_group_id:
            raise ValueError(f"Expected a non-empty value for `user_group_id` but received {user_group_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/iam/user_groups/{user_group_id}",
                account_id=account_id,
                user_group_id=user_group_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[UserGroupGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[UserGroupGetResponse]], ResultWrapper[UserGroupGetResponse]),
        )


class UserGroupsResourceWithRawResponse:
    def __init__(self, user_groups: UserGroupsResource) -> None:
        self._user_groups = user_groups

        self.create = to_raw_response_wrapper(
            user_groups.create,
        )
        self.update = to_raw_response_wrapper(
            user_groups.update,
        )
        self.list = to_raw_response_wrapper(
            user_groups.list,
        )
        self.delete = to_raw_response_wrapper(
            user_groups.delete,
        )
        self.get = to_raw_response_wrapper(
            user_groups.get,
        )

    @cached_property
    def members(self) -> MembersResourceWithRawResponse:
        return MembersResourceWithRawResponse(self._user_groups.members)


class AsyncUserGroupsResourceWithRawResponse:
    def __init__(self, user_groups: AsyncUserGroupsResource) -> None:
        self._user_groups = user_groups

        self.create = async_to_raw_response_wrapper(
            user_groups.create,
        )
        self.update = async_to_raw_response_wrapper(
            user_groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            user_groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            user_groups.delete,
        )
        self.get = async_to_raw_response_wrapper(
            user_groups.get,
        )

    @cached_property
    def members(self) -> AsyncMembersResourceWithRawResponse:
        return AsyncMembersResourceWithRawResponse(self._user_groups.members)


class UserGroupsResourceWithStreamingResponse:
    def __init__(self, user_groups: UserGroupsResource) -> None:
        self._user_groups = user_groups

        self.create = to_streamed_response_wrapper(
            user_groups.create,
        )
        self.update = to_streamed_response_wrapper(
            user_groups.update,
        )
        self.list = to_streamed_response_wrapper(
            user_groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            user_groups.delete,
        )
        self.get = to_streamed_response_wrapper(
            user_groups.get,
        )

    @cached_property
    def members(self) -> MembersResourceWithStreamingResponse:
        return MembersResourceWithStreamingResponse(self._user_groups.members)


class AsyncUserGroupsResourceWithStreamingResponse:
    def __init__(self, user_groups: AsyncUserGroupsResource) -> None:
        self._user_groups = user_groups

        self.create = async_to_streamed_response_wrapper(
            user_groups.create,
        )
        self.update = async_to_streamed_response_wrapper(
            user_groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            user_groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            user_groups.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            user_groups.get,
        )

    @cached_property
    def members(self) -> AsyncMembersResourceWithStreamingResponse:
        return AsyncMembersResourceWithStreamingResponse(self._user_groups.members)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SmartRoutingEditResponse"]


class SmartRoutingEditResponse(BaseModel):
    id: str
    """Specifies the identifier of the Argo Smart Routing setting."""

    editable: bool
    """Specifies if the setting is editable."""

    value: Literal["on", "off"]
    """Specifies the enablement value of Argo Smart Routing."""

    modified_on: Optional[datetime] = None
    """Specifies the time when the setting was last modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .keys import (
    KeysResource,
    AsyncKeysResource,
    KeysResourceWithRawResponse,
    AsyncKeysResourceWithRawResponse,
    KeysResourceWithStreamingResponse,
    AsyncKeysResourceWithStreamingResponse,
)
from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)
from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .access import (
    AccessResource,
    AsyncAccessResource,
    AccessResourceWithRawResponse,
    AsyncAccessResourceWithRawResponse,
    AccessResourceWithStreamingResponse,
    AsyncAccessResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from .policies import (
    PoliciesResource,
    AsyncPoliciesResource,
    PoliciesResourceWithRawResponse,
    AsyncPoliciesResourceWithRawResponse,
    PoliciesResourceWithStreamingResponse,
    AsyncPoliciesResourceWithStreamingResponse,
)
from .bookmarks import (
    BookmarksResource,
    AsyncBookmarksResource,
    BookmarksResourceWithRawResponse,
    AsyncBookmarksResourceWithRawResponse,
    BookmarksResourceWithStreamingResponse,
    AsyncBookmarksResourceWithStreamingResponse,
)
from .gateway_ca import (
    GatewayCAResource,
    AsyncGatewayCAResource,
    GatewayCAResourceWithRawResponse,
    AsyncGatewayCAResourceWithRawResponse,
    GatewayCAResourceWithStreamingResponse,
    AsyncGatewayCAResourceWithStreamingResponse,
)
from .ai_controls import (
    AIControlsResource,
    AsyncAIControlsResource,
    AIControlsResourceWithRawResponse,
    AsyncAIControlsResourceWithRawResponse,
    AIControlsResourceWithStreamingResponse,
    AsyncAIControlsResourceWithStreamingResponse,
)
from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)
from .certificates import (
    CertificatesResource,
    AsyncCertificatesResource,
    CertificatesResourceWithRawResponse,
    AsyncCertificatesResourceWithRawResponse,
    CertificatesResourceWithStreamingResponse,
    AsyncCertificatesResourceWithStreamingResponse,
)
from .custom_pages import (
    CustomPagesResource,
    AsyncCustomPagesResource,
    CustomPagesResourceWithRawResponse,
    AsyncCustomPagesResourceWithRawResponse,
    CustomPagesResourceWithStreamingResponse,
    AsyncCustomPagesResourceWithStreamingResponse,
)
from .infrastructure import (
    InfrastructureResource,
    AsyncInfrastructureResource,
    InfrastructureResourceWithRawResponse,
    AsyncInfrastructureResourceWithRawResponse,
    InfrastructureResourceWithStreamingResponse,
    AsyncInfrastructureResourceWithStreamingResponse,
)
from .service_tokens import (
    ServiceTokensResource,
    AsyncServiceTokensResource,
    ServiceTokensResourceWithRawResponse,
    AsyncServiceTokensResourceWithRawResponse,
    ServiceTokensResourceWithStreamingResponse,
    AsyncServiceTokensResourceWithStreamingResponse,
)
from .saml_certificates import (
    SAMLCertificatesResource,
    AsyncSAMLCertificatesResource,
    SAMLCertificatesResourceWithRawResponse,
    AsyncSAMLCertificatesResourceWithRawResponse,
    SAMLCertificatesResourceWithStreamingResponse,
    AsyncSAMLCertificatesResourceWithStreamingResponse,
)

__all__ = [
    "AIControlsResource",
    "AsyncAIControlsResource",
    "AIControlsResourceWithRawResponse",
    "AsyncAIControlsResourceWithRawResponse",
    "AIControlsResourceWithStreamingResponse",
    "AsyncAIControlsResourceWithStreamingResponse",
    "GatewayCAResource",
    "AsyncGatewayCAResource",
    "GatewayCAResourceWithRawResponse",
    "AsyncGatewayCAResourceWithRawResponse",
    "GatewayCAResourceWithStreamingResponse",
    "AsyncGatewayCAResourceWithStreamingResponse",
    "SAMLCertificatesResource",
    "AsyncSAMLCertificatesResource",
    "SAMLCertificatesResourceWithRawResponse",
    "AsyncSAMLCertificatesResourceWithRawResponse",
    "SAMLCertificatesResourceWithStreamingResponse",
    "AsyncSAMLCertificatesResourceWithStreamingResponse",
    "InfrastructureResource",
    "AsyncInfrastructureResource",
    "InfrastructureResourceWithRawResponse",
    "AsyncInfrastructureResourceWithRawResponse",
    "InfrastructureResourceWithStreamingResponse",
    "AsyncInfrastructureResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
    "CertificatesResource",
    "AsyncCertificatesResource",
    "CertificatesResourceWithRawResponse",
    "AsyncCertificatesResourceWithRawResponse",
    "CertificatesResourceWithStreamingResponse",
    "AsyncCertificatesResourceWithStreamingResponse",
    "GroupsResource",
    "AsyncGroupsResource",
    "GroupsResourceWithRawResponse",
    "AsyncGroupsResourceWithRawResponse",
    "GroupsResourceWithStreamingResponse",
    "AsyncGroupsResourceWithStreamingResponse",
    "ServiceTokensResource",
    "AsyncServiceTokensResource",
    "ServiceTokensResourceWithRawResponse",
    "AsyncServiceTokensResourceWithRawResponse",
    "ServiceTokensResourceWithStreamingResponse",
    "AsyncServiceTokensResourceWithStreamingResponse",
    "BookmarksResource",
    "AsyncBookmarksResource",
    "BookmarksResourceWithRawResponse",
    "AsyncBookmarksResourceWithRawResponse",
    "BookmarksResourceWithStreamingResponse",
    "AsyncBookmarksResourceWithStreamingResponse",
    "KeysResource",
    "AsyncKeysResource",
    "KeysResourceWithRawResponse",
    "AsyncKeysResourceWithRawResponse",
    "KeysResourceWithStreamingResponse",
    "AsyncKeysResourceWithStreamingResponse",
    "LogsResource",
    "AsyncLogsResource",
    "LogsResourceWithRawResponse",
    "AsyncLogsResourceWithRawResponse",
    "LogsResourceWithStreamingResponse",
    "AsyncLogsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "CustomPagesResource",
    "AsyncCustomPagesResource",
    "CustomPagesResourceWithRawResponse",
    "AsyncCustomPagesResourceWithRawResponse",
    "CustomPagesResourceWithStreamingResponse",
    "AsyncCustomPagesResourceWithStreamingResponse",
    "TagsResource",
    "AsyncTagsResource",
    "TagsResourceWithRawResponse",
    "AsyncTagsResourceWithRawResponse",
    "TagsResourceWithStreamingResponse",
    "AsyncTagsResourceWithStreamingResponse",
    "PoliciesResource",
    "AsyncPoliciesResource",
    "PoliciesResourceWithRawResponse",
    "AsyncPoliciesResourceWithRawResponse",
    "PoliciesResourceWithStreamingResponse",
    "AsyncPoliciesResourceWithStreamingResponse",
    "AccessResource",
    "AsyncAccessResource",
    "AccessResourceWithRawResponse",
    "AsyncAccessResourceWithRawResponse",
    "AccessResourceWithStreamingResponse",
    "AsyncAccessResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["SCIMConfigAuthenticationOAuthBearerToken"]


class SCIMConfigAuthenticationOAuthBearerToken(BaseModel):
    """
    Attributes for configuring OAuth Bearer Token authentication scheme for SCIM provisioning to an application.
    """

    token: str
    """Token used to authenticate with the remote SCIM service."""

    scheme: Literal["oauthbearertoken"]
    """The authentication scheme to use when making SCIM requests to this application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.tunnels.cloudflared.token_get_response import TokenGetResponse

__all__ = ["TokenResource", "AsyncTokenResource"]


class TokenResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TokenResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TokenResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokenResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TokenResourceWithStreamingResponse(self)

    def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Gets the token used to associate cloudflared with a specific tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/cfd_tunnel/{tunnel_id}/token", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncTokenResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTokenResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokenResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokenResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTokenResourceWithStreamingResponse(self)

    async def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Gets the token used to associate cloudflared with a specific tunnel.

        Args:
          account_id: Cloudflare account ID

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/cfd_tunnel/{tunnel_id}/token", account_id=account_id, tunnel_id=tunnel_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TokenGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class TokenResourceWithRawResponse:
    def __init__(self, token: TokenResource) -> None:
        self._token = token

        self.get = to_raw_response_wrapper(
            token.get,
        )


class AsyncTokenResourceWithRawResponse:
    def __init__(self, token: AsyncTokenResource) -> None:
        self._token = token

        self.get = async_to_raw_response_wrapper(
            token.get,
        )


class TokenResourceWithStreamingResponse:
    def __init__(self, token: TokenResource) -> None:
        self._token = token

        self.get = to_streamed_response_wrapper(
            token.get,
        )


class AsyncTokenResourceWithStreamingResponse:
    def __init__(self, token: AsyncTokenResource) -> None:
        self._token = token

        self.get = async_to_streamed_response_wrapper(
            token.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["KolideInput"]


class KolideInput(BaseModel):
    connection_id: str
    """Posture Integration ID."""

    auth_state: Optional[List[Literal["Good", "Notified", "Will Block", "Blocked"]]] = None
    """The set of Kolide device authentication states that pass the posture check.

    Device must match one of the specified states.
    """

    count_operator: Optional[Literal["<", "<=", ">", ">=", "=="]] = FieldInfo(alias="countOperator", default=None)
    """Count Operator."""

    issue_count: Optional[str] = None
    """The Number of Issues."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DeviceNetwork", "Config"]


class Config(BaseModel):
    """
    The configuration object containing information for the WARP client to detect the managed network.
    """

    tls_sockaddr: str
    """
    A network address of the form "host:port" that the WARP client will use to
    detect the presence of a TLS host.
    """

    sha256: Optional[str] = None
    """
    The SHA-256 hash of the TLS certificate presented by the host found at
    tls_sockaddr. If absent, regular certificate verification (trusted roots, valid
    timestamp, etc) will be used to validate the certificate.
    """


class DeviceNetwork(BaseModel):
    config: Optional[Config] = None
    """
    The configuration object containing information for the WARP client to detect
    the managed network.
    """

    name: Optional[str] = None
    """The name of the device managed network. This name must be unique."""

    network_id: Optional[str] = None
    """API UUID."""

    type: Optional[Literal["tls"]] = None
    """The type of device managed network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.addressing.prefixes import service_binding_create_params
from ....types.addressing.prefixes.service_binding import ServiceBinding
from ....types.addressing.prefixes.service_binding_delete_response import ServiceBindingDeleteResponse

__all__ = ["ServiceBindingsResource", "AsyncServiceBindingsResource"]


class ServiceBindingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ServiceBindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ServiceBindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ServiceBindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ServiceBindingsResourceWithStreamingResponse(self)

    def create(
        self,
        prefix_id: str,
        *,
        account_id: str,
        cidr: str,
        service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceBinding]:
        """
        Creates a new Service Binding, routing traffic to IPs within the given CIDR to a
        service running on Cloudflare's network. **NOTE:** The first Service Binding
        created for an IP Prefix must exactly match the IP Prefix's CIDR. Subsequent
        Service Bindings may be created with a more-specific CIDR. Refer to the
        [Service Bindings Documentation](https://developers.cloudflare.com/byoip/service-bindings/)
        for compatibility details.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          cidr: IP Prefix in Classless Inter-Domain Routing format.

          service_id: Identifier of a Service on the Cloudflare network. Available services and their
              IDs may be found in the **List Services** endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=maybe_transform(
                {
                    "cidr": cidr,
                    "service_id": service_id,
                },
                service_binding_create_params.ServiceBindingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ServiceBinding]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ServiceBinding]], ResultWrapper[ServiceBinding]),
        )

    def list(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ServiceBinding]:
        """List the Cloudflare services this prefix is currently bound to.

        Traffic sent to
        an address within an IP prefix will be routed to the Cloudflare service of the
        most-specific Service Binding matching the address. **Example:** binding
        `192.0.2.0/24` to Cloudflare Magic Transit and `192.0.2.1/32` to the Cloudflare
        CDN would route traffic for `192.0.2.1` to the CDN, and traffic for all other
        IPs in the prefix to Cloudflare Magic Transit.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            page=SyncSinglePage[ServiceBinding],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ServiceBinding,
        )

    def delete(
        self,
        binding_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceBindingDeleteResponse:
        """
        Delete a Service Binding

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          binding_id: Identifier of a Service Binding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not binding_id:
            raise ValueError(f"Expected a non-empty value for `binding_id` but received {binding_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings/{binding_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                binding_id=binding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceBindingDeleteResponse,
        )

    def get(
        self,
        binding_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceBinding]:
        """
        Fetch a single Service Binding

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          binding_id: Identifier of a Service Binding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not binding_id:
            raise ValueError(f"Expected a non-empty value for `binding_id` but received {binding_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings/{binding_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                binding_id=binding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ServiceBinding]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ServiceBinding]], ResultWrapper[ServiceBinding]),
        )


class AsyncServiceBindingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncServiceBindingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncServiceBindingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncServiceBindingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncServiceBindingsResourceWithStreamingResponse(self)

    async def create(
        self,
        prefix_id: str,
        *,
        account_id: str,
        cidr: str,
        service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceBinding]:
        """
        Creates a new Service Binding, routing traffic to IPs within the given CIDR to a
        service running on Cloudflare's network. **NOTE:** The first Service Binding
        created for an IP Prefix must exactly match the IP Prefix's CIDR. Subsequent
        Service Bindings may be created with a more-specific CIDR. Refer to the
        [Service Bindings Documentation](https://developers.cloudflare.com/byoip/service-bindings/)
        for compatibility details.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          cidr: IP Prefix in Classless Inter-Domain Routing format.

          service_id: Identifier of a Service on the Cloudflare network. Available services and their
              IDs may be found in the **List Services** endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            body=await async_maybe_transform(
                {
                    "cidr": cidr,
                    "service_id": service_id,
                },
                service_binding_create_params.ServiceBindingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ServiceBinding]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ServiceBinding]], ResultWrapper[ServiceBinding]),
        )

    def list(
        self,
        prefix_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ServiceBinding, AsyncSinglePage[ServiceBinding]]:
        """List the Cloudflare services this prefix is currently bound to.

        Traffic sent to
        an address within an IP prefix will be routed to the Cloudflare service of the
        most-specific Service Binding matching the address. **Example:** binding
        `192.0.2.0/24` to Cloudflare Magic Transit and `192.0.2.1/32` to the Cloudflare
        CDN would route traffic for `192.0.2.1` to the CDN, and traffic for all other
        IPs in the prefix to Cloudflare Magic Transit.

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings",
                account_id=account_id,
                prefix_id=prefix_id,
            ),
            page=AsyncSinglePage[ServiceBinding],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ServiceBinding,
        )

    async def delete(
        self,
        binding_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ServiceBindingDeleteResponse:
        """
        Delete a Service Binding

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          binding_id: Identifier of a Service Binding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not binding_id:
            raise ValueError(f"Expected a non-empty value for `binding_id` but received {binding_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings/{binding_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                binding_id=binding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ServiceBindingDeleteResponse,
        )

    async def get(
        self,
        binding_id: str,
        *,
        account_id: str,
        prefix_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ServiceBinding]:
        """
        Fetch a single Service Binding

        Args:
          account_id: Identifier of a Cloudflare account.

          prefix_id: Identifier of an IP Prefix.

          binding_id: Identifier of a Service Binding.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not prefix_id:
            raise ValueError(f"Expected a non-empty value for `prefix_id` but received {prefix_id!r}")
        if not binding_id:
            raise ValueError(f"Expected a non-empty value for `binding_id` but received {binding_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/addressing/prefixes/{prefix_id}/bindings/{binding_id}",
                account_id=account_id,
                prefix_id=prefix_id,
                binding_id=binding_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ServiceBinding]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ServiceBinding]], ResultWrapper[ServiceBinding]),
        )


class ServiceBindingsResourceWithRawResponse:
    def __init__(self, service_bindings: ServiceBindingsResource) -> None:
        self._service_bindings = service_bindings

        self.create = to_raw_response_wrapper(
            service_bindings.create,
        )
        self.list = to_raw_response_wrapper(
            service_bindings.list,
        )
        self.delete = to_raw_response_wrapper(
            service_bindings.delete,
        )
        self.get = to_raw_response_wrapper(
            service_bindings.get,
        )


class AsyncServiceBindingsResourceWithRawResponse:
    def __init__(self, service_bindings: AsyncServiceBindingsResource) -> None:
        self._service_bindings = service_bindings

        self.create = async_to_raw_response_wrapper(
            service_bindings.create,
        )
        self.list = async_to_raw_response_wrapper(
            service_bindings.list,
        )
        self.delete = async_to_raw_response_wrapper(
            service_bindings.delete,
        )
        self.get = async_to_raw_response_wrapper(
            service_bindings.get,
        )


class ServiceBindingsResourceWithStreamingResponse:
    def __init__(self, service_bindings: ServiceBindingsResource) -> None:
        self._service_bindings = service_bindings

        self.create = to_streamed_response_wrapper(
            service_bindings.create,
        )
        self.list = to_streamed_response_wrapper(
            service_bindings.list,
        )
        self.delete = to_streamed_response_wrapper(
            service_bindings.delete,
        )
        self.get = to_streamed_response_wrapper(
            service_bindings.get,
        )


class AsyncServiceBindingsResourceWithStreamingResponse:
    def __init__(self, service_bindings: AsyncServiceBindingsResource) -> None:
        self._service_bindings = service_bindings

        self.create = async_to_streamed_response_wrapper(
            service_bindings.create,
        )
        self.list = async_to_streamed_response_wrapper(
            service_bindings.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            service_bindings.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            service_bindings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PrebuiltPolicyListParams"]


class PrebuiltPolicyListParams(TypedDict, total=False):
    account_id: Required[str]

    destination_type: Literal["NONE", "ZERO_TRUST_LIST"]
    """Specify type of destination, omit to return all."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cache import (
    CacheResource,
    AsyncCacheResource,
    CacheResourceWithRawResponse,
    AsyncCacheResourceWithRawResponse,
    CacheResourceWithStreamingResponse,
    AsyncCacheResourceWithStreamingResponse,
)
from .variants import (
    VariantsResource,
    AsyncVariantsResource,
    VariantsResourceWithRawResponse,
    AsyncVariantsResourceWithRawResponse,
    VariantsResourceWithStreamingResponse,
    AsyncVariantsResourceWithStreamingResponse,
)
from .cache_reserve import (
    CacheReserveResource,
    AsyncCacheReserveResource,
    CacheReserveResourceWithRawResponse,
    AsyncCacheReserveResourceWithRawResponse,
    CacheReserveResourceWithStreamingResponse,
    AsyncCacheReserveResourceWithStreamingResponse,
)
from .smart_tiered_cache import (
    SmartTieredCacheResource,
    AsyncSmartTieredCacheResource,
    SmartTieredCacheResourceWithRawResponse,
    AsyncSmartTieredCacheResourceWithRawResponse,
    SmartTieredCacheResourceWithStreamingResponse,
    AsyncSmartTieredCacheResourceWithStreamingResponse,
)
from .origin_cloud_regions import (
    OriginCloudRegionsResource,
    AsyncOriginCloudRegionsResource,
    OriginCloudRegionsResourceWithRawResponse,
    AsyncOriginCloudRegionsResourceWithRawResponse,
    OriginCloudRegionsResourceWithStreamingResponse,
    AsyncOriginCloudRegionsResourceWithStreamingResponse,
)
from .regional_tiered_cache import (
    RegionalTieredCacheResource,
    AsyncRegionalTieredCacheResource,
    RegionalTieredCacheResourceWithRawResponse,
    AsyncRegionalTieredCacheResourceWithRawResponse,
    RegionalTieredCacheResourceWithStreamingResponse,
    AsyncRegionalTieredCacheResourceWithStreamingResponse,
)

__all__ = [
    "CacheReserveResource",
    "AsyncCacheReserveResource",
    "CacheReserveResourceWithRawResponse",
    "AsyncCacheReserveResourceWithRawResponse",
    "CacheReserveResourceWithStreamingResponse",
    "AsyncCacheReserveResourceWithStreamingResponse",
    "SmartTieredCacheResource",
    "AsyncSmartTieredCacheResource",
    "SmartTieredCacheResourceWithRawResponse",
    "AsyncSmartTieredCacheResourceWithRawResponse",
    "SmartTieredCacheResourceWithStreamingResponse",
    "AsyncSmartTieredCacheResourceWithStreamingResponse",
    "VariantsResource",
    "AsyncVariantsResource",
    "VariantsResourceWithRawResponse",
    "AsyncVariantsResourceWithRawResponse",
    "VariantsResourceWithStreamingResponse",
    "AsyncVariantsResourceWithStreamingResponse",
    "RegionalTieredCacheResource",
    "AsyncRegionalTieredCacheResource",
    "RegionalTieredCacheResourceWithRawResponse",
    "AsyncRegionalTieredCacheResourceWithRawResponse",
    "RegionalTieredCacheResourceWithStreamingResponse",
    "AsyncRegionalTieredCacheResourceWithStreamingResponse",
    "OriginCloudRegionsResource",
    "AsyncOriginCloudRegionsResource",
    "OriginCloudRegionsResourceWithRawResponse",
    "AsyncOriginCloudRegionsResourceWithRawResponse",
    "OriginCloudRegionsResourceWithStreamingResponse",
    "AsyncOriginCloudRegionsResourceWithStreamingResponse",
    "CacheResource",
    "AsyncCacheResource",
    "CacheResourceWithRawResponse",
    "AsyncCacheResourceWithRawResponse",
    "CacheResourceWithStreamingResponse",
    "AsyncCacheResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["SummaryGetResponse", "User"]


class User(BaseModel):
    email: str

    event_count: int

    last_event: datetime

    max_risk_level: Literal["low", "medium", "high"]

    name: str

    user_id: str


class SummaryGetResponse(BaseModel):
    users: List[User]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CertificateUpdateParams"]


class CertificateUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    custom_hostname_id: Required[str]
    """Identifier."""

    certificate_pack_id: Required[str]
    """Identifier."""

    custom_certificate: Required[str]
    """If a custom uploaded certificate is used."""

    custom_key: Required[str]
    """The key for a custom uploaded certificate."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PipelineListParams"]


class PipelineListParams(TypedDict, total=False):
    account_id: Required[str]
    """Specifies the public ID of the account."""

    page: str
    """Specifies which page to retrieve."""

    per_page: str
    """Specifies the number of pipelines per page."""

    search: str
    """Specifies the prefix of pipeline name to search."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionParam"]


class ActionParam(TypedDict, total=False):
    """Actions pattern."""

    type: Required[Literal["drop", "forward", "worker"]]
    """Type of supported action."""

    value: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .scope_param import ScopeParam

__all__ = ["RouteCreateParams"]


class RouteCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    nexthop: Required[str]
    """The next-hop IP Address for the static route."""

    prefix: Required[str]
    """IP Prefix in Classless Inter-Domain Routing format."""

    priority: Required[int]
    """Priority of the static route."""

    description: str
    """An optional human provided description of the static route."""

    scope: ScopeParam
    """Used only for ECMP routes."""

    weight: int
    """Optional weight of the ECMP scope - if provided."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .setting_edit_params import SettingEditParams as SettingEditParams
from .universal_ssl_settings import UniversalSSLSettings as UniversalSSLSettings



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["BulkGetParams"]


class BulkGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    domain: SequenceNotStr[str]
    """Accepts multiple values like `?domain=cloudflare.com&domain=example.com`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Mirage"]


class Mirage(BaseModel):
    id: Optional[Literal["mirage"]] = None
    """
    Cloudflare Mirage reduces bandwidth used by images in mobile browsers. It can
    accelerate loading of image-heavy websites on very slow mobile connections and
    HTTP/1.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Mirage."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .asn import (
    ASNResource,
    AsyncASNResource,
    ASNResourceWithRawResponse,
    AsyncASNResourceWithRawResponse,
    ASNResourceWithStreamingResponse,
    AsyncASNResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .configs.configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)

__all__ = ["BotnetFeedResource", "AsyncBotnetFeedResource"]


class BotnetFeedResource(SyncAPIResource):
    @cached_property
    def asn(self) -> ASNResource:
        return ASNResource(self._client)

    @cached_property
    def configs(self) -> ConfigsResource:
        return ConfigsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BotnetFeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BotnetFeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BotnetFeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BotnetFeedResourceWithStreamingResponse(self)


class AsyncBotnetFeedResource(AsyncAPIResource):
    @cached_property
    def asn(self) -> AsyncASNResource:
        return AsyncASNResource(self._client)

    @cached_property
    def configs(self) -> AsyncConfigsResource:
        return AsyncConfigsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBotnetFeedResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBotnetFeedResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBotnetFeedResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBotnetFeedResourceWithStreamingResponse(self)


class BotnetFeedResourceWithRawResponse:
    def __init__(self, botnet_feed: BotnetFeedResource) -> None:
        self._botnet_feed = botnet_feed

    @cached_property
    def asn(self) -> ASNResourceWithRawResponse:
        return ASNResourceWithRawResponse(self._botnet_feed.asn)

    @cached_property
    def configs(self) -> ConfigsResourceWithRawResponse:
        return ConfigsResourceWithRawResponse(self._botnet_feed.configs)


class AsyncBotnetFeedResourceWithRawResponse:
    def __init__(self, botnet_feed: AsyncBotnetFeedResource) -> None:
        self._botnet_feed = botnet_feed

    @cached_property
    def asn(self) -> AsyncASNResourceWithRawResponse:
        return AsyncASNResourceWithRawResponse(self._botnet_feed.asn)

    @cached_property
    def configs(self) -> AsyncConfigsResourceWithRawResponse:
        return AsyncConfigsResourceWithRawResponse(self._botnet_feed.configs)


class BotnetFeedResourceWithStreamingResponse:
    def __init__(self, botnet_feed: BotnetFeedResource) -> None:
        self._botnet_feed = botnet_feed

    @cached_property
    def asn(self) -> ASNResourceWithStreamingResponse:
        return ASNResourceWithStreamingResponse(self._botnet_feed.asn)

    @cached_property
    def configs(self) -> ConfigsResourceWithStreamingResponse:
        return ConfigsResourceWithStreamingResponse(self._botnet_feed.configs)


class AsyncBotnetFeedResourceWithStreamingResponse:
    def __init__(self, botnet_feed: AsyncBotnetFeedResource) -> None:
        self._botnet_feed = botnet_feed

    @cached_property
    def asn(self) -> AsyncASNResourceWithStreamingResponse:
        return AsyncASNResourceWithStreamingResponse(self._botnet_feed.asn)

    @cached_property
    def configs(self) -> AsyncConfigsResourceWithStreamingResponse:
        return AsyncConfigsResourceWithStreamingResponse(self._botnet_feed.configs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OrganizationUpdateParams", "Parent", "Profile"]


class OrganizationUpdateParams(TypedDict, total=False):
    name: Required[str]

    parent: Parent

    profile: Profile


class Parent(TypedDict, total=False):
    id: Required[str]


class Profile(TypedDict, total=False):
    business_address: Required[str]

    business_email: Required[str]

    business_name: Required[str]

    business_phone: Required[str]

    external_metadata: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .exclude_update_params import ExcludeUpdateParams as ExcludeUpdateParams
from .include_update_params import IncludeUpdateParams as IncludeUpdateParams
from .certificate_edit_params import CertificateEditParams as CertificateEditParams
from .fallback_domain_update_params import FallbackDomainUpdateParams as FallbackDomainUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import date
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.radar import agent_readiness_summary_params
from ..._base_client import make_request_options
from ...types.radar.agent_readiness_summary_response import AgentReadinessSummaryResponse

__all__ = ["AgentReadinessResource", "AsyncAgentReadinessResource"]


class AgentReadinessResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentReadinessResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AgentReadinessResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentReadinessResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AgentReadinessResourceWithStreamingResponse(self)

    def summary(
        self,
        dimension: Literal["CHECK"],
        *,
        date: Union[str, date] | Omit = omit,
        domain_category: SequenceNotStr[str] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentReadinessSummaryResponse:
        """
        Returns a summary of AI agent readiness scores across scanned domains, grouped
        by the specified dimension. Data is sourced from weekly bulk scans. All values
        are raw domain counts.

        Args:
          dimension: Specifies the agent readiness data dimension by which to group the results.

          date: Filters results by the specified date.

          domain_category: Filters results by domain category.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return self._get(
            path_template("/radar/agent_readiness/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "domain_category": domain_category,
                        "format": format,
                        "name": name,
                    },
                    agent_readiness_summary_params.AgentReadinessSummaryParams,
                ),
                post_parser=ResultWrapper[AgentReadinessSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[AgentReadinessSummaryResponse], ResultWrapper[AgentReadinessSummaryResponse]),
        )


class AsyncAgentReadinessResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentReadinessResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentReadinessResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentReadinessResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAgentReadinessResourceWithStreamingResponse(self)

    async def summary(
        self,
        dimension: Literal["CHECK"],
        *,
        date: Union[str, date] | Omit = omit,
        domain_category: SequenceNotStr[str] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AgentReadinessSummaryResponse:
        """
        Returns a summary of AI agent readiness scores across scanned domains, grouped
        by the specified dimension. Data is sourced from weekly bulk scans. All values
        are raw domain counts.

        Args:
          dimension: Specifies the agent readiness data dimension by which to group the results.

          date: Filters results by the specified date.

          domain_category: Filters results by domain category.

          format: Format in which results will be returned.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not dimension:
            raise ValueError(f"Expected a non-empty value for `dimension` but received {dimension!r}")
        return await self._get(
            path_template("/radar/agent_readiness/summary/{dimension}", dimension=dimension),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "date": date,
                        "domain_category": domain_category,
                        "format": format,
                        "name": name,
                    },
                    agent_readiness_summary_params.AgentReadinessSummaryParams,
                ),
                post_parser=ResultWrapper[AgentReadinessSummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[AgentReadinessSummaryResponse], ResultWrapper[AgentReadinessSummaryResponse]),
        )


class AgentReadinessResourceWithRawResponse:
    def __init__(self, agent_readiness: AgentReadinessResource) -> None:
        self._agent_readiness = agent_readiness

        self.summary = to_raw_response_wrapper(
            agent_readiness.summary,
        )


class AsyncAgentReadinessResourceWithRawResponse:
    def __init__(self, agent_readiness: AsyncAgentReadinessResource) -> None:
        self._agent_readiness = agent_readiness

        self.summary = async_to_raw_response_wrapper(
            agent_readiness.summary,
        )


class AgentReadinessResourceWithStreamingResponse:
    def __init__(self, agent_readiness: AgentReadinessResource) -> None:
        self._agent_readiness = agent_readiness

        self.summary = to_streamed_response_wrapper(
            agent_readiness.summary,
        )


class AsyncAgentReadinessResourceWithStreamingResponse:
    def __init__(self, agent_readiness: AsyncAgentReadinessResource) -> None:
        self._agent_readiness = agent_readiness

        self.summary = async_to_streamed_response_wrapper(
            agent_readiness.summary,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ApplicationType"]

ApplicationType: TypeAlias = Literal[
    "self_hosted",
    "saas",
    "ssh",
    "vnc",
    "app_launcher",
    "warp",
    "biso",
    "bookmark",
    "dash_sso",
    "infrastructure",
    "rdp",
    "mcp",
    "mcp_portal",
    "proxy_endpoint",
]
